#   Copyright (c) 2021 Works Applications Co., Ltd.
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
import os
import unittest

import sudachipy
from sudachipy import Dictionary


class PosMatcherTestCase(unittest.TestCase):

    def setUp(self):
        resource_dir = os.path.join(os.path.dirname(
            os.path.abspath(__file__)), 'resources')
        self.dict = Dictionary(os.path.join(
            resource_dir, 'sudachi.json'), resource_dir)
        self.tokenizer_obj = self.dict.create()

    def test_create_fn(self):
        m = self.dict.pos_matcher(lambda p: p[0] == "名詞")
        self.assertIsNotNone(m)

    def test_create_declarative(self):
        m = self.dict.pos_matcher([("名詞",)])
        self.assertEqual(list(m), [('名詞', '固有名詞', '地名', '一般', '*', '*'),
                                   ('名詞', '普通名詞', '一般', '*', '*', '*'),
                                   ('名詞', '数詞', '*', '*', '*', '*')])

    def test_create_declarative_empty(self):
        m = self.dict.pos_matcher([()])
        self.assertEqual(len(m), 9)

    def test_iter(self):
        m = self.dict.pos_matcher(lambda p: p[0] != "名詞")
        self.assertEqual(list(m), [('助動詞', '*', '*', '*', '助動詞-タ', '終止形-一般'),
                                   ('助詞', '接続助詞', '*', '*', '*', '*'),
                                   ('助詞', '格助詞', '*', '*', '*', '*'),
                                   ('動詞', '非自立可能', '*', '*', '五段-カ行', '終止形-一般'),
                                   ('動詞', '非自立可能', '*', '*', '五段-カ行', '連用形-促音便'),
                                   ('被子植物門', '双子葉植物綱', 'ムクロジ目', 'ミカン科', 'ミカン属', 'スダチ')])

    def test_skips(self):
        m = self.dict.pos_matcher([(None, None, None, None, None, "終止形-一般")])
        self.assertEqual(list(m), [('助動詞', '*', '*', '*', '助動詞-タ', '終止形-一般'),
                                   ('動詞', '非自立可能', '*', '*', '五段-カ行', '終止形-一般')])

    def test_match_nouns(self):
        tok = self.dict.create(mode=sudachipy.SplitMode.A)
        m = self.dict.pos_matcher([("名詞",)])
        data = [x.surface() for x in tok.tokenize("東京に行く") if m(x)]
        self.assertEqual(data, ["東京"])

    def test_union(self):
        m1 = self.dict.pos_matcher(lambda p: p[0] == "名詞")
        m2 = self.dict.pos_matcher(lambda p: p[0] == "動詞")
        m3 = m1 | m2
        self.assertEqual(len(m1) + len(m2), len(m3))

    def test_intersection(self):
        m1 = self.dict.pos_matcher(lambda p: p[5] == "終止形-一般")
        m2 = self.dict.pos_matcher(lambda p: p[0] == "動詞")
        m3 = m1 & m2
        self.assertEqual(1, len(m3))
        self.assertEqual(list(m3), [('動詞', '非自立可能', '*', '*', '五段-カ行', '終止形-一般')])

    def test_difference(self):
        m1 = self.dict.pos_matcher(lambda p: p[5] == "終止形-一般")
        m2 = self.dict.pos_matcher(lambda p: p[0] == "動詞")
        m3 = m1 - m2
        self.assertEqual(1, len(m3))
        self.assertEqual(list(m3), [('助動詞', '*', '*', '*', '助動詞-タ', '終止形-一般')])

    def test_invert(self):
        m1 = self.dict.pos_matcher(lambda p: p[5] == "終止形-一般")
        self.assertEqual(len(m1), 2)
        self.assertEqual(len(~m1), 7)


if __name__ == '__main__':
    unittest.main()



# Copyright (c) 2019 Works Applications Co., Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import unittest

from sudachipy import Dictionary, SplitMode


class TestTokenizer(unittest.TestCase):

    def setUp(self):
        resource_dir = os.path.join(os.path.dirname(
            os.path.abspath(__file__)), 'resources')
        self.dict_ = Dictionary(os.path.join(
            resource_dir, 'sudachi.json'), resource_dir)
        self.tokenizer_obj = self.dict_.create()

    def test_nothing(self):
        pass

    def test_tokenize_small_katanana_only(self):
        ms = self.tokenizer_obj.tokenize('ァ')
        self.assertEqual(1, len(ms))

    def test_part_of_speech(self):
        ms = self.tokenizer_obj.tokenize('京都')
        self.assertEqual(1, len(ms))
        m = ms[0]
        pid = m.part_of_speech_id()
        self.assertEqual(3, pid)
        pos = m.part_of_speech()
        self.assertEqual(('名詞', '固有名詞', '地名', '一般', '*', '*'), pos)

    def test_get_word_id(self):
        ms = self.tokenizer_obj.tokenize('京都')
        self.assertEqual(1, len(ms))
        self.assertEqual(('名詞', '固有名詞', '地名', '一般', '*', '*'),
                         ms[0].part_of_speech())

        wid = ms[0].word_id()
        ms = self.tokenizer_obj.tokenize('ぴらる')
        self.assertEqual(1, len(ms))
        self.assertNotEqual(wid, ms[0].word_id())
        self.assertEqual(('名詞', '普通名詞', '一般', '*', '*', '*'),
                         ms[0].part_of_speech())

        ms = self.tokenizer_obj.tokenize('京')
        self.assertEqual(1, len(ms))

    def test_get_dictionary_id(self):
        ms = self.tokenizer_obj.tokenize('京都')
        self.assertEqual(1, ms.size())
        self.assertEqual(0, ms[0].dictionary_id())

        ms = self.tokenizer_obj.tokenize('ぴらる')
        self.assertEqual(1, ms.size())
        self.assertEqual(1, ms[0].dictionary_id())

        ms = self.tokenizer_obj.tokenize('京')
        self.assertEqual(1, ms.size())
        self.assertTrue(ms[0].dictionary_id() < 0)

    def test_get_synonym_group_ids(self):
        ms = self.tokenizer_obj.tokenize('京都')
        self.assertEqual(1, ms.size())
        self.assertEqual([1, 5], ms[0].synonym_group_ids())

        ms = self.tokenizer_obj.tokenize('ぴらる')
        self.assertEqual(1, ms.size())
        self.assertEqual([], ms[0].synonym_group_ids())

        ms = self.tokenizer_obj.tokenize('東京府')
        self.assertEqual(1, ms.size())
        self.assertEqual([1, 3], ms[0].synonym_group_ids())

    def test_tokenize_kanji_alphabet_word(self):
        self.assertEqual(len(self.tokenizer_obj.tokenize('特a')), 1)
        self.assertEqual(len(self.tokenizer_obj.tokenize('ab')), 1)
        self.assertEqual(len(self.tokenizer_obj.tokenize('特ab')), 2)

    def test_tokenizer_with_dots(self):
        ms = self.tokenizer_obj.tokenize('京都…')
        self.assertEqual(4, ms.size())
        self.assertEqual(ms[1].surface(), '…')
        self.assertEqual(ms[1].normalized_form(), '.')
        self.assertEqual(ms[2].surface(), '')
        self.assertEqual(ms[2].normalized_form(), '.')
        self.assertEqual(ms[3].surface(), '')
        self.assertEqual(ms[3].normalized_form(), '.')

    def test_tokenizer_morpheme_split(self):
        ms = self.tokenizer_obj.tokenize('東京都', SplitMode.C)
        self.assertEqual(1, ms.size())
        self.assertEqual(ms[0].surface(), '東京都')

        ms_a = ms[0].split(SplitMode.A)
        self.assertEqual(2, ms_a.size())
        self.assertEqual(ms_a[0].surface(), '東京')
        self.assertEqual(ms_a[1].surface(), '都')

    def test_tokenizer_morpheme_list_range(self):
        ms = self.tokenizer_obj.tokenize('東京都', SplitMode.A)
        self.assertEqual(2, ms.size())
        self.assertEqual(ms[0].surface(), '東京')
        self.assertEqual(ms[1].surface(), '都')

        self.assertEqual(ms[-1].surface(), ms[1].surface())
        self.assertEqual(ms[-2].surface(), ms[0].surface())
        with self.assertRaises(IndexError):
            ms[2]
        with self.assertRaises(IndexError):
            ms[-3]

    def test_tokenizer_subset(self):
        ms1 = self.tokenizer_obj.tokenize('東京都')

        tok = self.dict_.create(fields={"pos"})
        ms2 = tok.tokenize('東京都')
        self.assertEqual(ms1[0].part_of_speech_id(), ms2[0].part_of_speech_id())

    def test_tokenizer_out_param(self):
        ms1 = self.tokenizer_obj.tokenize('東京都東京府')
        m = ms1[0]
        self.assertEqual(m.surface(), '東京都')

        ms2 = self.tokenizer_obj.tokenize('すだち', out=ms1)
        self.assertEqual(id(ms1), id(ms2))
        self.assertEqual(m.surface(), 'すだち')


if __name__ == '__main__':
    unittest.main()



# Copyright (c) 2019 Works Applications Co., Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import unittest

from sudachipy import Dictionary


class TestTokenizer(unittest.TestCase):

    def setUp(self):
        resource_dir = os.path.join(os.path.dirname(
            os.path.abspath(__file__)), 'resources')
        self.dict_ = Dictionary(os.path.join(
            resource_dir, 'sudachi.json'), resource_dir)
        self.tokenizer_obj = self.dict_.create()

    def test_wordinfo(self):
        # た
        wi = self.tokenizer_obj.tokenize('た')[0].get_word_info()
        self.assertEqual('た', wi.surface)
        self.assertEqual(3, wi.head_word_length)
        self.assertEqual(0, wi.pos_id)
        self.assertEqual('た', wi.normalized_form)
        self.assertEqual(-1, wi.dictionary_form_word_id)
        self.assertEqual('た', wi.dictionary_form)
        self.assertEqual('タ', wi.reading_form)
        self.assertEqual([], wi.a_unit_split)
        self.assertEqual([], wi.b_unit_split)
        self.assertEqual([], wi.word_structure)

        # 行っ
        wi = self.tokenizer_obj.tokenize('行っ')[0].get_word_info()
        self.assertEqual('行っ', wi.surface)
        self.assertEqual('行く', wi.normalized_form)
        self.assertEqual(7, wi.dictionary_form_word_id)
        self.assertEqual('行く', wi.dictionary_form)

        # 東京都
        wi = self.tokenizer_obj.tokenize('東京都')[0].get_word_info()
        self.assertEqual('東京都', wi.surface)
        self.assertEqual([5, 9], wi.a_unit_split)
        self.assertEqual([], wi.b_unit_split)
        self.assertEqual([5, 9], wi.word_structure)
        self.assertEqual([], wi.synonym_group_ids)

    def test_wordinfo_with_longword(self):
        s = "0123456789" * 30
        wi = self.tokenizer_obj.tokenize(s)[0].get_word_info()
        self.assertEqual(300, len(wi.surface))
        self.assertEqual(300, wi.head_word_length)
        self.assertEqual(300, len(wi.normalized_form))
        self.assertEqual(-1, wi.dictionary_form_word_id)
        self.assertEqual(300, len(wi.dictionary_form))
        self.assertEqual(570, len(wi.reading_form))

    def test_wordinfo_surface(self):
        wi = self.tokenizer_obj.tokenize('京都')[0].get_word_info()
        self.assertEqual(wi.surface, "京都")

        wi = self.tokenizer_obj.tokenize('東京府')[0].get_word_info()
        self.assertEqual(wi.surface, "東京府")

    def test_wordinfo_length(self):
        wi = self.tokenizer_obj.tokenize('京都')[0].get_word_info()
        self.assertEqual(wi.head_word_length, 6)
        self.assertEqual(wi.length(), 6)

        wi = self.tokenizer_obj.tokenize('東京府')[0].get_word_info()
        self.assertEqual(wi.head_word_length, 9)
        self.assertEqual(wi.length(), 9)

    def test_wordinfo_pos(self):
        wi = self.tokenizer_obj.tokenize('東')[0].get_word_info()
        self.assertEqual(wi.pos_id, 4)

        wi = self.tokenizer_obj.tokenize('東京府')[0].get_word_info()
        self.assertEqual(wi.pos_id, 3)

    def test_wordinfo_forms(self):
        wi = self.tokenizer_obj.tokenize('東京')[0].get_word_info()
        self.assertEqual(wi.dictionary_form_word_id, -1)
        self.assertEqual(wi.dictionary_form, '東京')
        self.assertEqual(wi.normalized_form, '東京')
        self.assertEqual(wi.reading_form, 'トウキョウ')

        wi = self.tokenizer_obj.tokenize('東京府')[0].get_word_info()
        self.assertEqual(wi.dictionary_form_word_id, -1)
        self.assertEqual(wi.dictionary_form, "東京府")
        self.assertEqual(wi.normalized_form, "東京府")
        self.assertEqual(wi.reading_form, "トウキョウフ")

    def test_wordinfo_unit_split(self):
        wi = self.tokenizer_obj.tokenize('東京')[0].get_word_info()
        self.assertEqual(wi.a_unit_split, [])
        self.assertEqual(wi.b_unit_split, [])

        wi = self.tokenizer_obj.tokenize('東京府')[0].get_word_info()
        self.assertEqual(wi.a_unit_split, [5, 2**28 + 1])
        self.assertEqual(wi.b_unit_split, [])

    def test_wordinfo_word_structure(self):
        wi = self.tokenizer_obj.tokenize('東京')[0].get_word_info()
        self.assertEqual(wi.word_structure, [])

        wi = self.tokenizer_obj.tokenize('東京府')[0].get_word_info()
        self.assertEqual(wi.word_structure, [5, 2**28 + 1])

    def test_wordinfo_synonym_group_ids(self):
        wi = self.tokenizer_obj.tokenize('東京')[0].get_word_info()
        self.assertEqual(wi.synonym_group_ids, [])

        wi = self.tokenizer_obj.tokenize('東京府')[0].get_word_info()
        self.assertEqual(wi.synonym_group_ids, [1, 3])


if __name__ == '__main__':
    unittest.main()



# Copyright (c) 2019 Works Applications Co., Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import unittest

from sudachipy import Dictionary, SplitMode


class TestTokenizer(unittest.TestCase):

    def setUp(self):
        resource_dir = os.path.join(os.path.dirname(
            os.path.abspath(__file__)), 'resources')
        self.dict_ = Dictionary(os.path.join(
            resource_dir, 'sudachi.json'), resource_dir)
        self.tokenizer_obj = self.dict_.create()

    def test_empty_list(self):
        ms = self.tokenizer_obj.tokenize('')
        self.assertEqual(0, ms.size())

    def test_morpheme_split(self):
        ms = self.tokenizer_obj.tokenize('東京都', SplitMode.C)
        self.assertEqual(1, ms.size())
        self.assertEqual(ms[0].surface(), '東京都')

        ms_a = ms[0].split(SplitMode.A)
        self.assertEqual(2, ms_a.size())
        self.assertEqual(ms_a[0].surface(), '東京')
        self.assertEqual(ms_a[1].surface(), '都')

    def test_morpheme_split_middle(self):
        ms_c = self.tokenizer_obj.tokenize('京都東京都京都', SplitMode.C)
        self.assertEqual(3, ms_c.size())
        self.assertEqual(ms_c[1].surface(), '東京都')
        self.assertEqual(ms_c[1].begin(), 2)
        self.assertEqual(ms_c[1].end(), 5)

        ms_a = ms_c[1].split(SplitMode.A)
        self.assertEqual(2, ms_a.size())
        self.assertEqual(ms_a[0].surface(), '東京')
        self.assertEqual(ms_a[0].begin(), 2)
        self.assertEqual(ms_a[0].end(), 4)
        self.assertEqual(ms_a[1].surface(), '都')
        self.assertEqual(ms_a[1].begin(), 4)
        self.assertEqual(ms_a[1].end(), 5)

    def test_morpheme_index(self):
        m = self.tokenizer_obj.tokenize('東京都')[0]
        self.assertEqual(m.begin(), 0)
        self.assertEqual(m.end(), 3)

    def test_morpheme_pos(self):
        m = self.tokenizer_obj.tokenize('京都')[0]
        self.assertEqual(m.part_of_speech_id(), 3)
        self.assertEqual(m.part_of_speech(), (
                         '名詞', '固有名詞', '地名', '一般', '*', '*'))

    def test_morpheme_forms(self):
        m = self.tokenizer_obj.tokenize('東京')[0]
        self.assertEqual(m.surface(), '東京')
        self.assertEqual(m.dictionary_form(), '東京')
        self.assertEqual(m.normalized_form(), '東京')
        self.assertEqual(m.reading_form(), 'トウキョウ')

        m = self.tokenizer_obj.tokenize('ぴらる')[0]
        self.assertEqual(m.surface(), 'ぴらる')
        self.assertEqual(m.dictionary_form(), 'ぴらる')
        self.assertEqual(m.normalized_form(), 'ぴらる')
        self.assertEqual(m.reading_form(), 'ピラル')

    def test_morpheme_dictionary_id(self):
        m = self.tokenizer_obj.tokenize('京都')[0]
        self.assertEqual(m.dictionary_id(), 0)

        m = self.tokenizer_obj.tokenize('ぴらる')[0]
        self.assertEqual(m.dictionary_id(), 1)

        m = self.tokenizer_obj.tokenize('京')[0]
        self.assertTrue(m.dictionary_id() < 0)

    def test_morpheme_word_id(self):
        m = self.tokenizer_obj.tokenize('京都')[0]
        self.assertEqual(m.word_id(), 3)

        m = self.tokenizer_obj.tokenize('ぴらる')[0]
        self.assertEqual(m.word_id(), 2**28 + 0)

    def test_morpheme_oov(self):
        m = self.tokenizer_obj.tokenize('京都')[0]
        self.assertEqual(m.is_oov(), False)

        m = self.tokenizer_obj.tokenize('京')[0]
        self.assertEqual(m.is_oov(), True)

    def test_morpheme_synonym_group_ids(self):
        m = self.tokenizer_obj.tokenize('京都')[0]
        self.assertEqual(m.synonym_group_ids(), [1, 5])

        m = self.tokenizer_obj.tokenize('ぴらる')[0]
        self.assertEqual(m.synonym_group_ids(), [])

        m = self.tokenizer_obj.tokenize('東京府')[0]
        self.assertEqual(m.synonym_group_ids(), [1, 3])

    def test_normalize_half_full(self):
        m = self.tokenizer_obj.tokenize('特Ａ東京')
        self.assertEqual(len(m), 2)

        self.assertEqual(m[0].surface(), "特Ａ")
        self.assertEqual(m[0].begin(), 0)
        self.assertEqual(m[0].end(), 2)

    def test_normalize_katakana_half_full(self):
        m = self.tokenizer_obj.tokenize('ｶﾞ5')
        self.assertEqual(len(m), 2)

        self.assertEqual(m[0].surface(), "ｶﾞ")
        self.assertEqual(m[0].end(), 2)
        self.assertEqual(m[1].begin(), 2)
        self.assertEqual(len(m[0]), 2)

    def test_morpheme_split_out(self):
        ms = self.tokenizer_obj.tokenize('東京都', SplitMode.C)
        self.assertEqual(1, ms.size())
        self.assertEqual(ms[0].surface(), '東京都')

        ms_a = ms[0].split(SplitMode.A, out=None)
        self.assertEqual(2, ms_a.size())
        self.assertEqual(ms_a[0].surface(), '東京')
        self.assertEqual(ms_a[1].surface(), '都')

        ms = self.tokenizer_obj.tokenize("京都東京都京都", SplitMode.C)
        ms_b = ms[1].split(SplitMode.A, out=ms_a)
        self.assertEqual(id(ms_a), id(ms_b))
        self.assertEqual(ms_a[0].surface(), '東京')
        self.assertEqual(ms_a[1].surface(), '都')

    def test_morpheme_str_repr(self):
        ms = self.tokenizer_obj.tokenize('東京都', SplitMode.A)
        self.assertEqual(2, ms.size())
        self.assertEqual(str(ms), '東京 都')
        self.assertEqual(repr(ms), '<MorphemeList[\n  <Morpheme(東京, 0:2, (0, 5))>,\n  <Morpheme(都, 2:3, (0, 9))>,\n]>')
        self.assertEqual(str(ms[0]), '東京')
        self.assertEqual(str(ms[1]), '都')
        self.assertEqual(repr(ms[0]), '<Morpheme(東京, 0:2, (0, 5))>')
        self.assertEqual(repr(ms[1]), '<Morpheme(都, 2:3, (0, 9))>')


if __name__ == '__main__':
    unittest.main()



#   Copyright (c) 2023 Works Applications Co., Ltd.
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

import unittest

from sudachipy import Dictionary
from sudachipy.config import Config

class MyTestCase(unittest.TestCase):
    def setUp(self) -> None:
        self.dict = Dictionary(config=Config(projection="reading"))

    def test_projection_surface_override(self):
        tok = self.dict.create(projection="surface")
        morphs = tok.tokenize("酒を飲む人")
        self.assertEqual(4, morphs.size())
        self.assertEqual("酒", morphs[0].surface())
        self.assertEqual("を", morphs[1].surface())
        self.assertEqual("飲む", morphs[2].surface())
        self.assertEqual("人", morphs[3].surface())

    def test_projection_reading(self):
        tok = self.dict.create()
        morphs = tok.tokenize("酒を飲む人")
        self.assertEqual(4, morphs.size())
        self.assertEqual("サケ", morphs[0].surface())
        self.assertEqual("ヲ", morphs[1].surface())
        self.assertEqual("ノム", morphs[2].surface())
        self.assertEqual("ヒト", morphs[3].surface())

    def test_projection_dictionary(self):
        tok = self.dict.create(projection="dictionary")
        morphs = tok.tokenize("酒を飲まなかった人")
        self.assertEqual(6, morphs.size())
        self.assertEqual("酒", morphs[0].surface())
        self.assertEqual("を", morphs[1].surface())
        self.assertEqual("飲む", morphs[2].surface())
        self.assertEqual("ない", morphs[3].surface())
        self.assertEqual("た", morphs[4].surface())
        self.assertEqual("人", morphs[5].surface())

    def test_projection_normalized(self):
        tok = self.dict.create(projection="normalized")
        morphs = tok.tokenize("MEGAへ行く")
        self.assertEqual(3, morphs.size())
        self.assertEqual("メガ", morphs[0].surface())
        self.assertEqual("MEGA", morphs[0].raw_surface())
        self.assertEqual("へ", morphs[1].surface())
        self.assertEqual("行く", morphs[2].surface())

    def test_projection_dictionary_and_surface(self):
        tok = self.dict.create(projection="dictionary_and_surface")
        morphs = tok.tokenize("酒を飲まなかった人")
        self.assertEqual(6, morphs.size())
        self.assertEqual("酒", morphs[0].surface())
        self.assertEqual("を", morphs[1].surface())
        self.assertEqual("飲ま", morphs[2].surface())
        self.assertEqual("なかっ", morphs[3].surface())
        self.assertEqual("た", morphs[4].surface())
        self.assertEqual("人", morphs[5].surface())

    def test_projection_normalized_and_surface(self):
        tok = self.dict.create(projection="normalized_and_surface")
        morphs = tok.tokenize("MEGAへ行こう")
        self.assertEqual(3, morphs.size())
        self.assertEqual("メガ", morphs[0].surface())
        self.assertEqual("MEGA", morphs[0].raw_surface())
        self.assertEqual("へ", morphs[1].surface())
        self.assertEqual("行こう", morphs[2].surface())

    def test_projection_normalized_nouns(self):
        tok = self.dict.create(projection="normalized_nouns")
        morphs = tok.tokenize("MEGAへ行こう")
        self.assertEqual(3, morphs.size())
        self.assertEqual("メガ", morphs[0].surface())
        self.assertEqual("MEGA", morphs[0].raw_surface())
        self.assertEqual("へ", morphs[1].surface())
        self.assertEqual("行こう", morphs[2].surface())


if __name__ == '__main__':
    unittest.main()



#   Copyright (c) 2021 Works Applications Co., Ltd.
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

import unittest

import tokenizers
from tokenizers.models import WordLevel

import sudachipy
from sudachipy import MorphemeList
from sudachipy.config import Config


class PretokenizerTestCase(unittest.TestCase):
    def setUp(self) -> None:
        self.dict = sudachipy.Dictionary()

    def test_instantiates_default(self):
        pretok = self.dict.pre_tokenizer()
        self.assertIsNotNone(pretok)

    def test_works_in_simple_pipeline(self):
        pretok = self.dict.pre_tokenizer()
        vocab = {
            "[UNK]": 0,
            "京都": 1,
            "に": 2,
            "行く": 3
        }
        tok = tokenizers.Tokenizer(WordLevel(vocab, unk_token="[UNK]"))
        tok.pre_tokenizer = pretok
        res = tok.encode("京都へ行く")
        self.assertEqual(res.ids, [1, 0, 3])

    def test_works_with_different_split_mode(self):
        pretok = self.dict.pre_tokenizer(sudachipy.SplitMode.A)
        vocab = {
            "[UNK]": 0,
            "外国": 1,
            "参政": 2,
            "権": 3,
            "人": 5,
            "外国人参政権": 4
        }
        tok = tokenizers.Tokenizer(WordLevel(vocab, unk_token="[UNK]"))
        tok.pre_tokenizer = pretok
        res = tok.encode("外国人参政権")
        self.assertEqual(res.ids, [1, 5, 2, 3])

    def test_with_handler(self):
        def _handler(index, sentence: tokenizers.NormalizedString, ml: MorphemeList):
            return [tokenizers.NormalizedString(ml[0].part_of_speech()[0]), tokenizers.NormalizedString(str(len(ml)))]
        pretok = self.dict.pre_tokenizer(sudachipy.SplitMode.A, handler=_handler)
        vocab = {
            "[UNK]": 0,
            "名詞": 6,
            "4": 7,
        }
        tok = tokenizers.Tokenizer(WordLevel(vocab, unk_token="[UNK]"))
        tok.pre_tokenizer = pretok
        res = tok.encode("外国人参政権")
        self.assertEqual(res.ids, [6, 7])


    def test_with_projection(self):
        pretok = self.dict.pre_tokenizer(sudachipy.SplitMode.A, projection="reading")
        vocab = {
            "[UNK]": 0,
            "ノム": 1,
            "サケ": 2,
            "ヒト": 3,
            "ヲ": 5,
        }
        tok = tokenizers.Tokenizer(WordLevel(vocab, unk_token="[UNK]"))
        tok.pre_tokenizer = pretok
        res = tok.encode("酒を飲む人")
        self.assertEqual([2, 5, 1, 3], res.ids)

    def test_projection_surface_override(self):
        dictobj = sudachipy.Dictionary(config=sudachipy.config.Config(projection="reading"))
        pretok = dictobj.pre_tokenizer(sudachipy.SplitMode.A, projection="surface")
        vocab = {
            "[UNK]": 0,
            "サケ": 1,
            "ヒト": 2,
            "ノム": 3,
            "ヲ": 5,
            "外国人参政権": 4
        }
        tok = tokenizers.Tokenizer(WordLevel(vocab, unk_token="[UNK]"))
        tok.pre_tokenizer = pretok
        res = tok.encode("酒を飲む人")
        self.assertEqual(res.ids, [1, 5, 3, 2])


if __name__ == '__main__':
    unittest.main()



# Copyright (c) 2019 Works Applications Co., Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import unittest

import sudachipy
from sudachipy import Dictionary, Tokenizer


class TestDictionary(unittest.TestCase):

    def setUp(self):
        resource_dir = os.path.join(os.path.dirname(
            os.path.abspath(__file__)), 'resources')
        self.dict_ = Dictionary(os.path.join(
            resource_dir, 'sudachi.json'), resource_dir=resource_dir)

    def tearDown(self) -> None:
        self.dict_.close()

    def test_create(self):
        self.assertEqual(Tokenizer, type(self.dict_.create()))

    def test_pos_of(self):
        self.assertIsNotNone(self.dict_.pos_of(0))

    def test_repr(self):
        repr_str = repr(self.dict_)
        self.assertTrue(repr_str.startswith("<SudachiDictionary(system="))
        self.assertTrue(repr_str.endswith("user.dic.test])>"))

    def test_lookup(self):
        ms = self.dict_.lookup("東京都")
        self.assertEqual(1, len(ms))
        self.assertEqual("トウキョウト", ms[0].reading_form())
        self.assertEqual(0, ms[0].begin())
        self.assertEqual(3, ms[0].end())
        splits = ms[0].split(sudachipy.SplitMode.A)
        self.assertEqual(2, len(splits))
        ms = self.dict_.lookup("京都", out=ms)
        self.assertEqual(1, len(ms))
        self.assertEqual("キョウト", ms[0].reading_form())
        self.assertEqual(0, ms[0].begin())
        self.assertEqual(2, ms[0].end())


if __name__ == '__main__':
    unittest.main()



#   Copyright (c) 2021 Works Applications Co., Ltd.
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

import tempfile
import unittest
from pathlib import Path

import sudachipy
from sudachipy.config import Config
from dataclasses import replace

FILE_PATH = Path(__file__)
RESOURCES_PATH = FILE_PATH.parent / "resources"
CFG_TEMPLATE = Config(
    oovProviderPlugin=[
        { "class" : "com.worksap.nlp.sudachi.SimpleOovPlugin",
          "oovPOS" : [ "名詞", "普通名詞", "一般", "*", "*", "*" ],
          "leftId" : 8,
          "rightId" : 8,
          "cost" : 6000 }
    ]
)


class MyTestCase(unittest.TestCase):
    def setUp(self) -> None:
        self.tempfiles = []
        self.tmpdir = tempfile.mkdtemp("sudachi", "test")
        super().setUp()

    def tearDown(self) -> None:
        for f in self.tempfiles:
            p = Path(f)
            if p.exists():
                p.unlink()
        Path(self.tmpdir).rmdir()
        super().tearDown()

    def test_build_system(self):
        out_tmp = tempfile.mktemp(prefix="sudachi_sy", suffix=".dic", dir=self.tmpdir)
        self.tempfiles.append(out_tmp)
        stats = sudachipy.sudachipy.build_system_dic(
            matrix=RESOURCES_PATH / "matrix.def",
            lex=[RESOURCES_PATH / "lex.csv"],
            output=out_tmp
        )
        self.assertIsNotNone(stats)
        cfg = replace(CFG_TEMPLATE, system=out_tmp)
        dict = sudachipy.Dictionary(config_path=cfg)
        tok = dict.create()
        result = tok.tokenize("東京にいく")
        self.assertEqual(result.size(), 3)

    def test_build_user1(self):
        sys_dic = tempfile.mktemp(prefix="sudachi_sy", suffix=".dic", dir=self.tmpdir)
        self.tempfiles.append(sys_dic)
        sudachipy.sudachipy.build_system_dic(
            matrix=RESOURCES_PATH / "matrix.def",
            lex=[RESOURCES_PATH / "lex.csv"],
            output=sys_dic
        )
        u1_dic = tempfile.mktemp(prefix="sudachi_u1", suffix=".dic", dir=self.tmpdir)
        self.tempfiles.append(u1_dic)
        sudachipy.sudachipy.build_user_dic(
            system=sys_dic,
            lex=[RESOURCES_PATH / "user1.csv"],
            output=u1_dic
        )

        cfg = replace(CFG_TEMPLATE, system=sys_dic, user=[u1_dic])
        dict = sudachipy.Dictionary(config=cfg)
        tok = dict.create()
        result = tok.tokenize("すだちにいく")
        self.assertEqual(result.size(), 3)
        self.assertEqual(result[0].dictionary_id(), 1)

    def test_build_user2(self):
        sys_dic = tempfile.mktemp(prefix="sudachi_sy", suffix=".dic", dir=self.tmpdir)
        self.tempfiles.append(sys_dic)
        sudachipy.sudachipy.build_system_dic(
            matrix=RESOURCES_PATH / "matrix.def",
            lex=[RESOURCES_PATH / "lex.csv"],
            output=sys_dic
        )
        u1_dic = tempfile.mktemp(prefix="sudachi_u1", suffix=".dic", dir=self.tmpdir)
        self.tempfiles.append(u1_dic)
        sudachipy.sudachipy.build_user_dic(
            system=sys_dic,
            lex=[RESOURCES_PATH / "user1.csv"],
            output=u1_dic
        )

        u2_dic = tempfile.mktemp(prefix="sudachi_u2", suffix=".dic", dir=self.tmpdir)
        self.tempfiles.append(u2_dic)
        sudachipy.sudachipy.build_user_dic(
            system=sys_dic,
            lex=[RESOURCES_PATH / "user2.csv"],
            output=u2_dic
        )

        cfg = replace(CFG_TEMPLATE, system=sys_dic, user=[u1_dic, u2_dic])
        dict = sudachipy.Dictionary(config_path=cfg)
        tok = dict.create()
        result = tok.tokenize("かぼすにいく")
        self.assertEqual(result.size(), 3)
        self.assertEqual(result[0].dictionary_id(), 2)
        self.assertEqual(result[0].part_of_speech()[0], "被子植物門")


if __name__ == '__main__':
    unittest.main()



#   Copyright (c) 2023 Works Applications Co., Ltd.
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
import dataclasses
from dataclasses import dataclass, asdict
from json import dumps

@dataclass
class Config:
    """
    SudachiPy rich configuration object.

    Fields passed here will override the fields in the default configuration.
    """
    system: str = None
    """
    Path to dictionary or one of three strings: 'small', 'core', 'notcore'.    
    If the file with the specified path does not exist and is not one of three special values, raise an error.
    If you want to use dictionary with one of predefined names, use relative paths e.g. './core' instead of 'core'.
    
    If the value is one of three special values and there does not exist a file with the same name,
    we try to load the dictionary from the SudachiDict_{system} installed package.
    For example, for "core" we will try to load the dictionary from the installed SudachiDict_core package.
    """
    user: list[str] = None
    """
    Paths to user dictionaries, maximum number of user dictionaries is 14
    """
    projection: str = "surface"
    """
    Output the following field as the result of [Morpheme.surface()] instead of its value.
    This option works for pre-tokenizers created for a given dictionary as well.
    The original value is available as [Morpheme.raw_surface()].
    
    This option is created for chiTra integration.
    
    Available options:
    - surface
    - normalized
    - reading
    - dictionary 
    - dictionary_and_surface 
    - normalized_and_surface
    - normalized_nouns
    """

    connectionCostPlugin: list = None
    oovProviderPlugin: list = None
    pathRewritePlugin: list = None
    inputTextPlugin: list = None
    characterDefinitionFile: str = None

    def as_jsons(self):
        """
        Convert this Config object to the json string
        """
        return dumps(_filter_nulls(asdict(self)))

    def update(self, **changes):
        return dataclasses.replace(self, **changes)


def _filter_nulls(data: dict) -> dict:
    keys = list(data.keys())
    for key in keys:
        v = data[key]
        if v is None:
            del data[key]
    return data
